
import {map, filter, mergeMap} from 'rxjs/operators';
import { Injectable } from '@angular/core';
import {
  Router,
  CanActivate,
  ActivatedRouteSnapshot,
  RouterStateSnapshot,
  CanLoad, Route, UrlSegment
} from '@angular/router';
import {Observable} from 'rxjs';
import {Session} from '../../login/utils/helper.class';
import {LoginErrorCodes} from '../../login/utils/guardHelper.class';
import {CommunityService} from '../community/community.service';
import { EnvironmentSpecificService} from '../../utils/properties/environment-specific.service';
import {ConnectHelper} from '../connectHelper';

@Injectable()
export class ConnectAdminLoginGuard implements CanActivate, CanLoad {
  constructor(private router: Router,
              private communityService: CommunityService,
              private propertiesService: EnvironmentSpecificService) {}

  check(community: string, path: string): Observable<boolean> | boolean {
    let errorCode = LoginErrorCodes.NOT_LOGIN;
    if (Session.isLoggedIn()) {
      if (Session.isPortalAdministrator() || Session.isCommunityCurator()) {
        return true;
      } else {
        const obs = this.propertiesService.subscribeEnvironment().pipe(map(res => res),mergeMap(properties => {
          return this.communityService.isCommunityManager(properties, properties['communityAPI'] + community, Session.getUserEmail());
        }),);
        obs.pipe(filter(enabled => !enabled))
            .subscribe(() => this.router.navigate(['/user-info'], {
              queryParams: {
                'errorCode': errorCode,
                'redirectUrl': path
              }
            }));
        return obs;
      }
    } else {
      errorCode = LoginErrorCodes.NOT_LOGIN;
      this.router.navigate(['/user-info'], {queryParams: {'errorCode': errorCode, 'redirectUrl': path}});

      return false;
    }
  }

  canActivate(route: ActivatedRouteSnapshot, state: RouterStateSnapshot): Observable<boolean> | boolean {
    return this.check(route.queryParams['communityId'], state.url);
  }

  canLoad(route: Route, segments: UrlSegment[]): Observable<boolean> | Promise<boolean> | boolean {
    const path = '/' + route.path + document.location.search;
    return this.check(ConnectHelper.getCommunityFromPath(path), path);
  }
}
