import {Injectable} from '@angular/core';
import {HttpClient, HttpErrorResponse} from "@angular/common/http";
import {throwError} from 'rxjs';
import {SoftwareInfo} from '../../utils/entities/softwareInfo';
import {EnvProperties} from '../../utils/properties/env-properties';
import {ParsingFunctions} from '../landing-utils/parsingFunctions.class';
import {map} from "rxjs/operators";

@Injectable()
export class SoftwareService {

  constructor(private http: HttpClient ) {
    this.parsingFunctions = new ParsingFunctions();
  }

  public parsingFunctions: ParsingFunctions;
  softwareInfo: SoftwareInfo;

  getSoftwareInfo (id: string, properties:EnvProperties):any {

    let url = properties.searchAPIURLLAst+'software/'+id+"?format=json";
    let key = url;


    return this.http.get((properties.useCache)? (properties.cacheUrl+encodeURIComponent(url)): url)
                    //.map(res => <any> res.json())
                    .pipe(map(res => [res['result']['header']['dri:status'], res['result']['metadata']['oaf:entity']['oaf:result'], res]))
                    .pipe(map(res => [res[1],
                                 res[1]['title'],
                                 res[1]['rels']['rel'],
                                 res[1]['children'],
                                 res[1]['pid'],
                                 res[1]['subject'],
                                 res[1]['bestaccessright'],
                                 res[1]['collectedfrom'],
                                 res[1]['context'],
                                 //res[1]['resulttype'],
                                 res[0],
                                 res[1]['creator'],
                                 res[1]['language'],
                                 res[1]['country'],
                                 res[1]['programmingLanguage'],
                                 res[2],
                                res[1]['journal'] //15
                    ]))
                    .pipe(map(res => this.parseSoftwareInfo(res, properties)));
  }

  private handleError (error: HttpErrorResponse) {
  // in a real world app, we may send the error to some remote logging infrastructure
  // instead of just logging it to the console
    console.log(error);
    return throwError(error  || 'Server error');
  }

  parseSoftwareInfo (data: any, properties: EnvProperties):any {
    this.softwareInfo = new SoftwareInfo();
    this.softwareInfo.record = data[14];
    if(data[0] != null) {
      var date:string = (data[0].dateofacceptance)+""; // transform to string in case it is an integer
      this.softwareInfo.date  = (date && (date).indexOf('-') !== -1)?date.split('-')[0]:date;
      this.softwareInfo.dateofacceptance = data[0].dateofacceptance;
      this.softwareInfo.publisher = data[0].publisher;
      if(!Array.isArray(data[0].description)) {
        this.softwareInfo.description = String(data[0].description);
      } else {
        this.softwareInfo.description = String(data[0].description[0]);
      }
      this.softwareInfo.embargoEndDate = data[0].embargoenddate;
    }
    /*
    this.softwareInfo.title = {"name": "", "url": "", "accessMode": ""};
    if(data[0]['bestaccessright'] && data[0]['bestaccessright'] && data[0]['bestaccessright'].hasOwnProperty("classid")) {
      this.softwareInfo.title.accessMode = data[0]['bestaccessright'].classid;
    }
    if(data[1] != null) {
      if(Array.isArray(data[1])) {
        this.softwareInfo.title['name'] = data[1][0].content;
      } else {
        this.softwareInfo.title['name'] = data[1].content;
      }
    }
    */
    if(data[0]['bestaccessright'] && data[0]['bestaccessright'] && data[0]['bestaccessright'].hasOwnProperty("classid")) {
      this.softwareInfo.accessMode = data[0]['bestaccessright'].classid;
    }
    if(data[1] != null) {
      if(Array.isArray(data[1])) {
        this.softwareInfo.title = String(data[1][0].content);
      } else {
        this.softwareInfo.title = String(data[1].content);
      }
    }

    if(data[2] != null) {
      let relation;
      let length = data[2].length!=undefined ? data[2].length : 1;

      for(let i=0; i<length; i++) {
        relation = data[2].length!=undefined ? data[2][i] : data[2];
        if(relation.hasOwnProperty("to")) {
          if(relation['to'].class == "isProducedBy") {
            this.softwareInfo.fundedByProjects = this.parsingFunctions.parseFundingByProjects(this.softwareInfo.fundedByProjects, relation, this.softwareInfo.projectsProvenanceVocabulary);
          } else if(relation['to'].class == "isRelatedTo") {
            let provenanceAction: string;
            if(relation.provenanceaction in this.softwareInfo.researchResultsProvenanceVocabulary) {
              provenanceAction = this.softwareInfo.researchResultsProvenanceVocabulary[relation.provenanceaction];
            } else {
              provenanceAction = "Other"
            }

            this.softwareInfo.relatedResearchResults = this.parsingFunctions.parseRelatedResearchResults(this.softwareInfo.relatedResearchResults, relation, provenanceAction);

          } else if(relation['to'].class == "hasAmongTopNSimilarDocuments") {
            this.softwareInfo.similarResearchResults = this.parsingFunctions.parseSimilarResearchResults(this.softwareInfo.similarResearchResults, relation);
          }
        }
      }
    }

    if(data[3] != null) {
      if(data[3].hasOwnProperty("instance")) {
        this.softwareInfo.hostedBy_collectedFrom = new Array<{  "downloadName": string, "downloadUrl": string[],
                                                                "collectedName": string, "collectedId": string,
                                                                "accessMode": string[], "bestAccessMode": string,
                                                                "type": string, "year":string}>();

        this.softwareInfo.types = new Array<string>();
        let types = new Set<string>();

        let counter = 0;
        let instance;

        let length = data[3]['instance'].length!=undefined ? data[3]['instance'].length : 1;

        for(let i=0; i<length; i++) {
          instance = data[3]['instance'].length!=undefined ? data[3]['instance'][i] : data[3]['instance'];

          this.parsingFunctions.parseTypes(this.softwareInfo.types, types, instance);

          if(instance.hasOwnProperty("webresource")) {
            let url;
            if(instance['webresource'].length == undefined) {
              url = instance['webresource'].url;
            } else{
              url = instance['webresource'][0].url;
            }

            if(instance.hasOwnProperty("hostedby")) {
              counter = this.parsingFunctions.parseHostedBy_collectedFrom(this.softwareInfo.hostedBy_collectedFrom, instance, data[0], url, counter/*, this.softwareInfo.title*/, this.softwareInfo.accessMode);
            }
          }
        }
      }
    }

    if(data[4] != null) {
      this.softwareInfo.identifiers = this.parsingFunctions.parseIdentifiers(data[4]);
    }

    if(data[5] != null) {
      let subjectResults: [string[], Map<string, string[]>, Map<string, string[]>] = this.parsingFunctions.parseAllSubjects(data[5]);
      this.softwareInfo.subjects = subjectResults[0];
      this.softwareInfo.otherSubjects = subjectResults[1];
      this.softwareInfo.classifiedSubjects = subjectResults[2];
    }

    // null argument is for journal
    this.softwareInfo.hostedBy_collectedFrom = this.parsingFunctions.addPublisherToHostedBy_collectedFrom(
                this.softwareInfo.hostedBy_collectedFrom, this.softwareInfo.publisher,
                null, this.softwareInfo.identifiers/*, this.softwareInfo.title*/);

    if(data[8] != null) {
      this.softwareInfo.contexts = this.parsingFunctions.parseContexts(data[8]);
    }

    // if(data[9] != null && this.softwareInfo.type == undefined) {
    //   if(data[9].hasOwnProperty('classname')) {
    //     this.softwareInfo.type = data[9].classname;
    //   }
    // }

    if(data[9] != null && data[9] == "under curation") {
      this.softwareInfo.underCurationMessage = true;
    } else {
      this.softwareInfo.underCurationMessage = false;
    }

    if(data[10] != null) {
      if(this.softwareInfo.authors == undefined) {
        this.softwareInfo.authors = new Array<{"fullName": string, "orcid": string}>();
      }

      let authors = data[10];
      let length = Array.isArray(authors) ? authors.length : 1;

      for(let i=0; i<length; i++) {
        let author = Array.isArray(authors) ? authors[i] : authors;
        if(author) {
          /*if (author.ORCID && author.ORCID.indexOf(properties.orcidURL) != -1) {
            author.ORCID = author.ORCID.substr(properties.orcidURL.length);
          }*/
          this.softwareInfo['authors'][author.rank] = {"fullName": author.content, "orcid": author.ORCID};
        }
      }
      this.softwareInfo.authors = this.softwareInfo.authors.filter(function (item) {
        return (item != undefined && item.fullName != undefined);
      });
    }

    if(data[11] != null) {
      this.softwareInfo.languages = this.parsingFunctions.parseLanguages(data[11]);
    }
    if(data[12] != null) {
      this.softwareInfo.countries = this.parsingFunctions.parseCountries(data[12]);
    }

    if(data[13] != null) {
      this.softwareInfo.programmingLanguages = this.parsingFunctions.parseProgrammingLanguages(data[13]);
    }

    if(this.softwareInfo.relatedResearchResults) {
      let self = this;
      this.softwareInfo.relatedResearchResults.forEach(function (value, key, map) {
        self.softwareInfo.relatedResearchResults.set(key, self.parsingFunctions.sortByPercentage(value));
      });
    }
    this.softwareInfo.similarResearchResults = this.parsingFunctions.sortByPercentage(this.softwareInfo.similarResearchResults);
    if(data[15] != null) {
      this.softwareInfo.journal = {"journal": "", "issn": "", "lissn": "", "eissn": "", "issue": "", "volume": "", "start_page": "", "end_page": ""}

      this.softwareInfo.journal['journal'] = data[15].content;
      this.softwareInfo.journal['issn'] = data[15].issn;
      this.softwareInfo.journal['lissn'] = data[15].lissn;
      this.softwareInfo.journal['eissn'] = data[15].eissn;
      this.softwareInfo.journal['issue'] = data[15].iss;
      this.softwareInfo.journal['volume'] = data[15].vol;
      this.softwareInfo.journal['start_page'] = data[15].sp;
      this.softwareInfo.journal['end_page'] = data[15].ep;
    }
    return this.softwareInfo;
  }
}
