import {Component, Input, ViewChild} from '@angular/core';
import { ActivatedRoute} from '@angular/router';
import {Location} from '@angular/common';
import { Filter, Value} from '../searchUtils/searchHelperClasses.class';

import {SearchOrpsService} from '../../services/searchOrps.service';
import {SearchResult}     from '../../utils/entities/searchResult';
import { ErrorCodes} from '../../utils/properties/errorCodes';
import {ErrorMessagesComponent}    from '../../utils/errorMessages.component';
import {SearchFields, FieldDetails} from '../../utils/properties/searchFields';
import {SearchPageComponent } from '../searchUtils/searchPage.component';
import {SearchUtilsClass } from '../searchUtils/searchUtils.class';
import {DOI} from '../../utils/string-utils.class';
import{EnvProperties} from '../../utils/properties/env-properties';

@Component({
    selector: 'search-orps',
    template: `

    <search-page pageTitle="Search Other Research Products"
                  formPlaceholderText = "Search for Other Research Products"
                 type="other research products" entityType="other" [(filters)] = "filters"
                 [(results)] = "results"   [(searchUtils)] = "searchUtils"
                 [baseUrl] = "baseUrl" (queryChange)="queryChanged($event)"
                 [csvParams]="csvParams" csvPath="other" advancedSearchLink="/search/advanced/other"
                 [disableForms]="disableForms"
                 [loadPaging]="loadPaging"
                 [oldTotalResults]="oldTotalResults"
                 searchFormClass="orpsSearchForm"
                 [(openaireLink)]=openaireLink
                 [(advancedSearchParameters)]=advancedSearchParameters
                 [piwikSiteId]=piwikSiteId
                 [(connectCommunityId)]=connectCommunityId
                 [(sort)]=sort >
    </search-page>
    `
})

export class SearchOrpsComponent {
  private errorCodes: ErrorCodes;
  private errorMessages: ErrorMessagesComponent;
  @Input() piwikSiteId = null;
  public results =[];
  public filters: Filter[] =[];
  public baseUrl:string;
  @Input() openaireLink: string ;
  @Input() connectCommunityId: string;
  @Input() advancedSearchParameters ;
  public searchUtils:SearchUtilsClass = new SearchUtilsClass();
  private sub: any;
  private subResults: any;
  private searchFields:SearchFields = new SearchFields();
  public refineFields: string[] =  this.searchFields.RESULT_REFINE_FIELDS;
  public fieldIdsMap=this.searchFields.RESULT_FIELDS;
  private urlParams : Map<string, string>;
  private _location:Location;
  public csvParams: string;
  public disableForms: boolean = false;
  public loadPaging: boolean = true;
  public oldTotalResults: number = 0;
  pagingLimit = 0;
  public sort: boolean = true;
properties: EnvProperties;
  @ViewChild (SearchPageComponent) searchPage : SearchPageComponent ;
  constructor (private route: ActivatedRoute, private _searchOrpsService: SearchOrpsService ) {

    this.errorCodes = new ErrorCodes();
    this.errorMessages = new ErrorMessagesComponent();
    this.searchUtils.status = this.errorCodes.LOADING;
    this.searchUtils.page =1;

  }

  public ngOnInit() {
    this.route.data
      .subscribe((data: { envSpecific: EnvProperties }) => {
        this.properties = data.envSpecific;
        this.baseUrl = data.envSpecific.searchLinkToOrps;
        this.pagingLimit = data.envSpecific.pagingLimit;

      });
    this.searchPage.refineFields = this.refineFields;
     this.searchPage.fieldIdsMap = this.fieldIdsMap;
     this.searchPage.type = "other research products";
     var firstLoad =true;
    this.sub =  this.route.queryParams.subscribe(params => {
      if(params['page'] && this.searchUtils.page != params['page']) {
        this.loadPaging = false;
        this.oldTotalResults = this.searchUtils.totalResults;
      }

      this.searchUtils.keyword = (params['keyword']?params['keyword']:'');
      var refine = true;
      if(this.searchUtils.page != ((params['page']=== undefined)?1:+params['page']) && this.filters && !firstLoad){
        refine = false;
      }
      firstLoad = false;
      this.searchUtils.page = (params['page']=== undefined)?1:+params['page'];
      this.searchUtils.size = (params['size']=== undefined)?10:+params['size'];
      if(this.searchUtils.size != 5 && this.searchUtils.size != 10 && this.searchUtils.size != 20 && this.searchUtils.size != 50) {
        this.searchUtils.size = 10;
      }
      this.searchUtils.sortBy = (params['sortBy'])?params['sortBy']:'';
      if(this.searchUtils.sortBy && this.searchUtils.sortBy != "resultdateofacceptance,descending" && this.searchUtils.sortBy != "resultdateofacceptance,ascending") {
        this.searchUtils.sortBy = "";
      }

      this.searchPage.connectCommunityId = this.connectCommunityId;
      var queryParameters = this.searchPage.getQueryParametersFromUrl(params);
       this._getResults(queryParameters, refine, this.searchUtils.page, this.searchUtils.size, this.searchUtils.sortBy);

    });
  }

  public ngOnDestroy() {
    if(this.sub){
      this.sub.unsubscribe();
    }
    if(this.subResults){
      this.subResults.unsubscribe();
    }
  }


public getResultsForEntity(entity:string, id:string, page: number, size: number){
  var parameters = "";

  if(entity == "project") {
    parameters = "projects/"+id;
  }

  if(parameters != "") {

      this._searchOrpsService.searchOrpsForEntity(parameters, page, size, this.properties).subscribe(
          data => {
              this.searchUtils.totalResults = data[0];
              //console.info("search Other Research Products for "+entity+": [Parameters:"+parameters+" ]  [total results:"+this.searchUtils.totalResults+"]");
              this.results = data[1];

              this.searchUtils.status = this.errorCodes.DONE;
              if(this.searchUtils.totalResults == 0 ){
                this.searchUtils.status = this.errorCodes.NONE;
              }
          },
          err => {
              //console.log(err);
              this.handleError("Error getting other research products for "+entity+" with id: "+id, err);
              this.searchUtils.status = this.errorMessages.getErrorCode(err.status);

               //TODO check erros (service not available, bad request)
              /*if(err.status == '404') {
                this.searchUtils.status = this.errorCodes.NOT_FOUND;
              } else if(err.status == '500') {
                this.searchUtils.status = this.errorCodes.ERROR;
              } else {
                this.searchUtils.status = this.errorCodes.NOT_AVAILABLE;
              }*/
          }
      );
  }
}

public getResultsForDataproviders(id:string, resultsFrom:string, page: number, size: number){
  var parameters;
  if(resultsFrom == "collectedFrom") {
      parameters = "orps?fq=collectedfromdatasourceid exact "+'"'+id+'"';
  } else if(resultsFrom == "hostedBy") {
      parameters = "orps?fq=resulthostingdatasourceid exact "+'"'+id+'"';
  }

  if(parameters != "") {

      this._searchOrpsService.searchOrpsForDataproviders(parameters, page, size, this.properties).subscribe(
          data => {
              this.searchUtils.totalResults = data[0];
              //console.info("search Other Research Products for Dataproviders: [Parameters:"+parameters+" ]  [total results:"+this.searchUtils.totalResults+"]");
              this.results = data[1];

              this.searchUtils.status = this.errorCodes.DONE;
              if(this.searchUtils.totalResults == 0 ){
                this.searchUtils.status = this.errorCodes.NONE;
              }
          },
          err => {
              //console.log(err);
              this.handleError("Error getting other research products for content provider ("+resultsFrom+") with id: "+id, err);
              this.searchUtils.status = this.errorMessages.getErrorCode(err.status);

               //TODO check erros (service not available, bad request)
              /*if(err.status == '404') {
                this.searchUtils.status = this.errorCodes.NOT_FOUND;
              } else if(err.status == '500') {
                this.searchUtils.status = this.errorCodes.ERROR;
              } else {
                this.searchUtils.status = this.errorCodes.NOT_AVAILABLE;
              }*/
          }
      );
  }
}

public getResults(keyword:string,refine:boolean, page: number, size: number, sortBy: string){
  var parameters = "";
  if(keyword.length > 0){
    var DOIs:string[] = DOI.getDOIsFromString(keyword);
    var doisParams = "";
    for(var i =0 ;i < DOIs.length; i++){
      doisParams+=(doisParams.length > 0?"&":"")+'doi="'+ DOIs[i]+'"';
    }
    if(doisParams.length > 0){
        parameters +=  "&"+doisParams;
    }else{
      parameters = "q=" + keyword;
    }
  }
  this._getResults(parameters,refine,page,size,sortBy);
}
private _getResults(parameters:string,refine:boolean, page: number, size: number, sortBy: string){
  if(page > this.pagingLimit) {
    size=0;
  }
  if(page <= this.pagingLimit || this.searchUtils.status == this.errorCodes.LOADING) {
    this.csvParams = parameters;

    this.searchUtils.status = this.errorCodes.LOADING;
    this.disableForms = true;
    this.results = [];
    this.searchUtils.totalResults = 0;

    this.subResults = this._searchOrpsService.searchOrps(parameters,(refine)?this.searchPage.getRefineFieldsQuery():null, page, size, sortBy, this.searchPage.getFields(), this.properties).subscribe(
      data => {
          this.searchUtils.totalResults = data[0];
          //console.info("search Other Research Products: [Parameters:"+parameters+" ]  [total results:"+this.searchUtils.totalResults+"]");
          this.results = data[1];
          if(refine){
            this.filters = data[2];
          }
          this.searchPage.checkSelectedFilters(this.filters);
          this.searchPage.updateBaseUrlWithParameters(this.filters);

          this.searchUtils.status = this.errorCodes.DONE;
          if(this.searchUtils.totalResults == 0 ){
            this.searchUtils.status = this.errorCodes.NONE;
          }
          this.disableForms = false;

          if(this.searchUtils.status == this.errorCodes.DONE) {
            // Page out of limit!!!
            let totalPages:any = this.searchUtils.totalResults/(this.searchUtils.size);
            if(!(Number.isInteger(totalPages))) {
                totalPages = (parseInt(totalPages, 10) + 1);
            }
            if(totalPages < page) {
              this.searchUtils.totalResults = 0;
              this.searchUtils.status = this.errorCodes.OUT_OF_BOUND;
            }
          }
      },
      err => {
          //console.log(err);
          this.handleError("Error getting other research products", err);
          this.searchUtils.status = this.errorMessages.getErrorCode(err.status);

           //TODO check erros (service not available, bad request)
          /*if(err.status == '404') {
            this.searchUtils.status = this.errorCodes.NOT_FOUND;
          } else if(err.status == '500') {
            this.searchUtils.status = this.errorCodes.ERROR;
          } else {
            this.searchUtils.status = this.errorCodes.NOT_AVAILABLE;
          }*/

          this.disableForms = false;
      }
    );
  }
}



  private setFilters(){
    //TODO set filters from
  }

  public queryChanged($event) {
    this.loadPaging = true;

    var parameters = $event.value;
    //this._getResults(parameters, true, this.searchUtils.page, this.searchUtils.size, this.searchUtils.sortBy);
  }

  private handleError(message: string, error) {
    console.error("Other Research Products simple Search Page: "+message, error);
  }
}
