
import {SearchPublicationsService} from '../../services/searchPublications.service';
import {ErrorCodes} from '../../utils/properties/errorCodes';
import {ErrorMessagesComponent}    from '../../utils/errorMessages.component';
import {SearchFields, FieldDetails} from '../../utils/properties/searchFields';
 import {SearchUtilsClass } from '../../searchPages/searchUtils/searchUtils.class';
import {DOI} from '../../utils/string-utils.class';
import {Subject} from 'rxjs';
import{EnvProperties} from '../../utils/properties/env-properties';
import {StringUtils} from '../../utils/string-utils.class';
export class FetchPublications {
  private errorCodes: ErrorCodes;
  private errorMessages: ErrorMessagesComponent;

  public results =[];

  public requestComplete: Subject<void>;

  // public filters =[];
  public searchUtils:SearchUtilsClass = new SearchUtilsClass();
  // public baseUrl:string = "";
  public sub: any;
  public subResults: any;
  public searchFields:SearchFields = new SearchFields();
  // public refineFields: string[] =  this.searchFields.RESULT_REFINE_FIELDS;
  // public fieldIdsMap=this.searchFields.RESULT_FIELDS;
    //:  { [key:string] :{ name:string, operator:string, type:string, indexField:string, equalityOperator:string  }} = this.searchFields.PUBLICATION_FIELDS_MAP;

  public CSV: any = {  "columnNames":  ["Title", "Authors", "Publication Year", "DOI",
                                       /*"Download From", "Publication type", "Journal",*/
                                       "Funder", "Project Name (GA Number)", "Access"],
                      "export":[]
                  };
  public CSVDownloaded = false;
  public csvParams: string;

  constructor ( private _searchPublicationsService: SearchPublicationsService ) {
    this.errorCodes = new ErrorCodes();
    this.errorMessages = new ErrorMessagesComponent();
    this.searchUtils.status = this.errorCodes.LOADING;

    this.requestComplete = new Subject<void>();
  }


  public ngOnDestroy() {
    if(this.sub){
      this.sub.unsubscribe();
    }
    if(this.subResults){
      this.subResults.unsubscribe();
    }
  }

  public getResultsByKeyword(keyword:string,  page: number, size: number, properties:EnvProperties, connectCommunityId=null){
    var parameters = "";
    if(keyword.length > 0){
      var DOIs:string[] = DOI.getDOIsFromString(keyword);
      var doisParams = "";

      for(var i =0 ;i < DOIs.length; i++){
        doisParams+=(doisParams.length > 0?"&":"")+'doi="'+ DOIs[i]+'"';
      }
      if(doisParams.length > 0){
        parameters +=  "&"+doisParams;
      }else{
        parameters = "q=" + StringUtils.URIEncode(keyword);
      }
    }


    //var errorCodes:ErrorCodes = new ErrorCodes();
    this.searchUtils.status = this.errorCodes.LOADING;
    var refineParams = (connectCommunityId)?("&fq="+StringUtils.URIEncode("communityId exact " + StringUtils.quote((connectCommunityId )))):null;
    this.subResults = this._searchPublicationsService.searchPublications(parameters,refineParams, page, size, "", [], properties).subscribe(
        data => {
            this.searchUtils.totalResults = data[0];
            //console.info("search Publications: [Parameters:"+parameters+" ]  [total results:"+this.searchUtils.totalResults+"]");
            this.results = data[1];

            //var errorCodes:ErrorCodes = new ErrorCodes();
            this.searchUtils.status = this.errorCodes.DONE;
            if(this.searchUtils.totalResults == 0 ){
              this.searchUtils.status = this.errorCodes.NONE;
            }
         },
        err => {
            /*console.log(err);
             //TODO check erros (service not available, bad request)
            // if( ){
            //   this.searchUtils.status = ErrorCodes.ERROR;
            // }
            //var errorCodes:ErrorCodes = new ErrorCodes();
            //this.searchUtils.status = errorCodes.ERROR;
            if(err.status == '404') {
              this.searchUtils.status = this.errorCodes.NOT_FOUND;
            } else if(err.status == '500') {
              this.searchUtils.status = this.errorCodes.ERROR;
            } else {
              this.searchUtils.status = this.errorCodes.NOT_AVAILABLE;
            }*/
            this.handleError("Error getting publications for keyword: "+keyword + (doisParams ? "(DOI)" : ""), err);
            this.searchUtils.status = this.errorMessages.getErrorCode(err.status);
        }
    );
  }

  public getNumForEntity(entity:string, id:string, properties:EnvProperties){
      //var errorCodes:ErrorCodes = new ErrorCodes();
      this.searchUtils.status = this.errorCodes.LOADING;


      if(id != "" && entity != "") {
          this._searchPublicationsService.numOfEntityPublications(id, entity, properties).subscribe(
              data => {
                  this.searchUtils.totalResults = data;

                  //var errorCodes:ErrorCodes = new ErrorCodes();
                  this.searchUtils.status = this.errorCodes.DONE;
                  if(this.searchUtils.totalResults == 0 ){
                    this.searchUtils.status = this.errorCodes.NONE;
                  }
              },
              err => {
                  /*console.log(err);
                   //TODO check erros (service not available, bad request)
                  // if( ){
                  //   this.searchUtils.status = ErrorCodes.ERROR;
                  // }
                  //var errorCodes:ErrorCodes = new ErrorCodes();
                  //this.searchUtils.status = errorCodes.ERROR;
                  if(err.status == '404') {
                    this.searchUtils.status = this.errorCodes.NOT_FOUND;
                  } else if(err.status == '500') {
                    this.searchUtils.status = this.errorCodes.ERROR;
                  } else {
                    this.searchUtils.status = this.errorCodes.NOT_AVAILABLE;
                  }*/
                  this.handleError("Error getting publications for "+entity+" with id: "+id, err);
                  this.searchUtils.status = this.errorMessages.getErrorCode(err.status);
              }
          );
      }
  }

public getResultsForEntity(entity:string, id:string, page: number, size: number, properties:EnvProperties){
    //var errorCodes:ErrorCodes = new ErrorCodes();
    this.searchUtils.status = this.errorCodes.LOADING;

  var parameters = "";
  if(entity == "project") {
    parameters = "projects/"+id;
  } else if(entity == "organization") {
    parameters = "organizations/"+id;
  }

  if(parameters != "") {
      this._searchPublicationsService.searchPublicationsForEntity(parameters, page, size, properties).subscribe(
          data => {
              this.searchUtils.totalResults = data[0];

              //console.info("search Publications for "+entity+": [Parameters:"+parameters+" ]  [total results:"+this.searchUtils.totalResults+"]");
              this.results = data[1];

              //var errorCodes:ErrorCodes = new ErrorCodes();
              this.searchUtils.status = this.errorCodes.DONE;
              if(this.searchUtils.totalResults == 0 ){
                this.searchUtils.status = this.errorCodes.NONE;
              }
          },
          err => {
              /*console.log(err);
               //TODO check erros (service not available, bad request)
              // if( ){
              //   this.searchUtils.status = ErrorCodes.ERROR;
              // }
              //var errorCodes:ErrorCodes = new ErrorCodes();
              //this.searchUtils.status = errorCodes.ERROR;

              if(err.status == '404') {
                this.searchUtils.status = this.errorCodes.NOT_FOUND;
              } else if(err.status == '500') {
                this.searchUtils.status = this.errorCodes.ERROR;
              } else {
                this.searchUtils.status = this.errorCodes.NOT_AVAILABLE;
              }*/
              this.handleError("Error getting publications for "+entity+" with id: "+id, err);
              this.searchUtils.status = this.errorMessages.getErrorCode(err.status);
          }
      );
  }
}

public getResultsForDataproviders(id:string, resultsFrom:string, page: number, size: number, properties:EnvProperties){
    //var errorCodes:ErrorCodes = new ErrorCodes();
    this.searchUtils.status = this.errorCodes.LOADING;

  var parameters;
  if(resultsFrom == "collectedFrom") {
      parameters = "publications?fq=collectedfromdatasourceid exact "+'"'+id+'"';
  } else if(resultsFrom == "hostedBy") {
      parameters = "publications?fq=resulthostingdatasourceid exact "+'"'+id+'"';
  }

  if(parameters != "") {

      this._searchPublicationsService.searchPublicationsForDataproviders(parameters, page, size, properties).subscribe(
          data => {
              this.searchUtils.totalResults = data[0];
              //console.info("search Publications for Dataproviders: [Parameters:"+parameters+" ]  [total results:"+this.searchUtils.totalResults+"]");
              this.results = data[1];

              //var errorCodes:ErrorCodes = new ErrorCodes();
              this.searchUtils.status = this.errorCodes.DONE;
              if(this.searchUtils.totalResults == 0 ){
                this.searchUtils.status = this.errorCodes.NONE;
              }
          },
          err => {
              /*console.log(err);
               //TODO check erros (service not available, bad request)
              // if( ){
              //   this.searchUtils.status = ErrorCodes.ERROR;
              // }
              //var errorCodes:ErrorCodes = new ErrorCodes();
              //this.searchUtils.status = errorCodes.ERROR;
              if(err.status == '404') {
                this.searchUtils.status = this.errorCodes.NOT_FOUND;
              } else if(err.status == '500') {
                this.searchUtils.status = this.errorCodes.ERROR;
              } else {
                this.searchUtils.status = this.errorCodes.NOT_AVAILABLE;
              }*/
              this.handleError("Error getting publications for content provider ("+resultsFrom+") with id: "+id, err);
              this.searchUtils.status = this.errorMessages.getErrorCode(err.status);
          }
      );
  }
}

public getAggregatorResults(id:string, page: number, size: number, properties:EnvProperties){
    //var errorCodes:ErrorCodes = new ErrorCodes();
    this.searchUtils.status = this.errorCodes.LOADING;

  this.subResults = this._searchPublicationsService.searchAggregators(id, '&fq=collectedfromdatasourceid exact "'+id+'"',"&refine=true&fields=resulthostingdatasource" , page, size, properties).subscribe(
      data => {
          this.results = data;
          this.searchUtils.totalResults = this.results.length;

          //var errorCodes:ErrorCodes = new ErrorCodes();
          this.searchUtils.status = this.errorCodes.DONE;
          if(this.searchUtils.totalResults == 0 ){
            this.searchUtils.status = this.errorCodes.NONE;
          }

          this.requestComplete.complete();
      },
      err => {
          /*console.log(err);
           //TODO check erros (service not available, bad request)
          // if( ){
          //   this.searchUtils.status = ErrorCodes.ERROR;
          // }
          //var errorCodes:ErrorCodes = new ErrorCodes();
          //this.searchUtils.status = errorCodes.ERROR;
          if(err.status == '404') {
            this.searchUtils.status = this.errorCodes.NOT_FOUND;
          } else if(err.status == '500') {
            this.searchUtils.status = this.errorCodes.ERROR;
          } else {
            this.searchUtils.status = this.errorCodes.NOT_AVAILABLE;
          }*/
          this.handleError("Error getting publications for aggregator with id: "+id, err);
          this.searchUtils.status = this.errorMessages.getErrorCode(err.status);

          this.requestComplete.complete();
      }
  );
}

  private handleError(message: string, error) {
    console.error("Fetch Publications (class): "+message, error);
  }
}
