import {Component, ViewEncapsulation, ElementRef, EventEmitter, Output, Input} from '@angular/core';

@Component({
  selector: 'modal-alert',
  template: `
  <div [class]="(!isOpen)?'uk-modal ':'uk-modal uk-open uk-animation-fade'" uk-modal  [open]="!isOpen" id="myModal" tabindex="-1" role="dialog" aria-labelledby="myModalLabel">
    <div class="uk-modal-dialog  uk-modal-body" role="">
         <div class="uk-modal-title" [hidden]=!alertHeader>
        <button class="uk-modal-close-default uk-float-right" (click)='cancel()' uk-close></button>
           <h4 class="modal-title text-center" id="myModalLabel">{{alertTitle}}</h4>
        </div>
        <div class="uk-margin  ">
          <div [hidden]=!alertMessage>
          {{message}}
          </div>
          <ng-content></ng-content>
        </div>
        <div *ngIf="okButtonLeft" class="uk-text-right" [hidden]=!alertFooter>
        <span [hidden]=!okButton >
          <button class="uk-button uk-button-default" (click)="ok()">{{okButtonText}}</button>
        </span>
        <span [hidden]=!cancelButton>
          <button class="uk-button uk-button-default" (click)="cancel()">{{cancelButtonText}}</button>
        </span>
        </div>
        <div *ngIf="!okButtonLeft" class="uk-text-right" [hidden]=!alertFooter>
         <span [hidden]=!cancelButton>
          <button class="uk-button uk-button-default" (click)="cancel()">{{cancelButtonText}}</button>
        </span>
        <span [hidden]=!okButton >
          <button *ngIf="okDisabled" class="uk-button uk-button-default" disabled>{{okButtonText}}</button>
          <button *ngIf="!okDisabled" class="uk-button portal-button" (click)="ok()">{{okButtonText}}</button>
        </span>
        </div>

    </div>
  </div>


  `,
  encapsulation: ViewEncapsulation.None,
})
/**
  * API to an open alert window.
  */
export class AlertModal{
  /**
     * Caption for the title.
     */
  public alertTitle:string;
  /**
     * Describes if the alert contains Ok Button.
     * The default Ok button will close the alert and emit the callback.
     * Defaults to true.
     */
  public okButton:boolean = true;
  /**
     * Caption for the OK button.
     * Default: Ok
     */
  public okButtonText:string= 'Ok';
  /**
     * Describes if the alert contains cancel Button.
     * The default Cancelbutton will close the alert.
     * Defaults to true.
     */
  public cancelButton:boolean = true;
  /**
     * Caption for the Cancel button.
     * Default: Cancel
     */
  public cancelButtonText:string = 'Cancel';
  /**
     * if the alertMessage is true it will show the contentString inside alert body.
     */
  public alertMessage:boolean = true;
  /**
     * Some message/content can be set in message which will be shown in alert body.
     */
  public message:string;
  /**
    * if the value is true alert footer will be visible or else it will be hidden.
    */
  public alertFooter:boolean= true;
  /**
    * shows alert header if the value is true.
    */
  public alertHeader:boolean = true;
  /**
    * if the value is true alert will be visible or else it will be hidden.
    */
  public isOpen:boolean=false;

  /**
   *  if the value is true ok button align on the left, else on the right
   */
  public okButtonLeft: boolean = true;

  /**
   *  if the value is true ok button is disabled
   */
  @Input()
  public okDisabled: boolean = false;
  /**
    * Emitted when a ok button was clicked
    * or when Ok method is called.
    */
  @Output() public alertOutput:EventEmitter<any> = new EventEmitter();
  constructor( public _elementRef: ElementRef){}
  /**
       * Opens a alert window creating backdrop.
       */
  open(){
    this.isOpen= true;
  }
  /**
     *  ok method closes the modal and emits modalOutput.
     */
  ok(){
    this.isOpen = false;
    this.alertOutput.emit(true);
  }
  /**
     *  cancel method closes the moda.
     */
  cancel(){
    this.isOpen = false;
  }
}
